@@ -89,7 +89,7 @@ local function make_tagged_room(options,chosen)
   -- screw with our attempts to understand and position the vault later; and hardwire transparency because lack of it can fail a whole layout
   -- TODO: Store these tags on the room first so we can actually support them down the line ...
   local old_tags = dgn.tags(mapdef)
-  dgn.tags(mapdef, "no_vmirror no_hmirror no_rotate transparent")
+  dgn.tags(mapdef, "no_vmirror no_hmirror no_rotate passable")
   -- Resolve the map so we can find its width / height
   local map, vplace = dgn.resolve_map(mapdef,false)
   local room,room_width,room_height
@@ -104,7 +104,9 @@ local function make_tagged_room(options,chosen)
   dgn.tags_remove(map, "no_vmirror no_hmirror no_rotate")
   -- restore the original tags to mapdef, since this state is persistent
   dgn.tags(mapdef, nil)
-  dgn.tags(mapdef, old_tags .. " transparent")
+  -- TODO: if a vault is not tagged passable, this addition is permanent; does
+  -- it affect anything?
+  dgn.tags(mapdef, old_tags .. " passable")
 
   local room_width,room_height = dgn.mapsize(map)
   local veto = false
@@ -174,9 +176,31 @@ local function pick_room(e, options)
     end
     options.did_ghost_chance = true
   end
+  -- Roll the chance to pick a Wizlab vault room if we haven't done so.
+  if not options.did_wizlab_chance then
+    if crawl.x_chance_in_y(dgn.wizlab_chance_percent, 100) then
+      for i, r in ipairs(options.room_type_weights) do
+        if r.generator == "tagged" and r.tag == "vaults_wizlab" then
+          chosen = r
+        end
+      end
+    end
+    options.did_wizlab_chance = true
+  end
+  -- Roll the chance to pick a Desolation vault room if we haven't done so.
+  if not options.did_desolation_chance then
+    if crawl.x_chance_in_y(dgn.desolation_chance_percent, 100) then
+      for i, r in ipairs(options.room_type_weights) do
+        if r.generator == "tagged" and r.tag == "vaults_desolation" then
+          chosen = r
+        end
+      end
+    end
+    options.did_desolation_chance = true
+  end
 
-  -- We aren't choosing a ghost vault, so pick a generator from the weighted
-  -- table.
+  -- We aren't choosing a ghost vault or a vault for wizlab or desolation
+  -- portals, so pick a generator from the weighted table.
   if chosen == nil then
     chosen = util.random_weighted_from(weight_callback,
                                        options.room_type_weights)
@@ -316,14 +340,14 @@ local function analyse_vault_post_placement(usage_grid,room,result,options)
     if dgn.in_bounds(p.x,p.y) and
       (feat.is_stone_stair(p.x,p.y)) or
       -- On V:1 the branch entrant stairs don't count as stone_stair; we need to check specifically for the V exit stairs
-      -- to avoid overwriting e.g. Crypt or Blade entrance stairs!
+      -- to avoid overwriting the Crypt entrance stairs!
       dgn.feature_name(dgn.grid(p.x,p.y)) == "exit_vaults" then
       -- Remove the stair and remember it
       dgn.grid(p.x,p.y,"floor") --TODO: Be more intelligent about how to replace it
       table.insert(result.stairs, { pos = { x = p.x, y = p.y } })
     end
     -- Substitute rock walls for surrounding wall type
-    if dgn.feature_name(dgn.grid(p.x,p.y)) == "rock_wall" then
+    if perform_subst and dgn.feature_name(dgn.grid(p.x,p.y)) == "rock_wall" then
       dgn.grid(p.x,p.y,room.wall_type)
     end
   end
@@ -347,7 +371,7 @@ function place_vaults_rooms(e, data, room_count, options)
         placed = true
         rooms_placed = rooms_placed + 1  -- Increment # rooms placed
         times_failed = 0 -- Reset fail count
-        -- Perform analysis for stairs (and perform inner wall substituion if applicable)
+        -- Perform analysis for stairs (and perform inner wall substitution if applicable)
         analyse_vault_post_placement(data,room,result,options)
         table.insert(results,result)
         -- Increment the count of rooms of this type
@@ -555,7 +579,7 @@ function vaults_maybe_place_vault(e, pos, usage_grid, usage, room, options)
     v_normal_dir = v_normal.dir
   end
 
-  -- If placing a room in an existing wall we have the normal stored alredy in the usage data
+  -- If placing a room in an existing wall we have the normal stored already in the usage data
   if usage.usage == "eligible" or usage.usage == "eligible_open" then
     v_normal = usage.normal
     v_normal_dir = usage.normal.dir
