@@ -2,7 +2,7 @@
 -- vault.lua: Vault helper functions from more than one file.
 --------------------------------------------------------------------------
 
--- Counting Pan runes for Ignacio and for exits.
+-- Counting Pan runes for exits.
 function count_pan_runes()
   local runes = 0
   for _, r in ipairs({"demonic","glowing","magical","fiery","dark"}) do
@@ -84,49 +84,56 @@ function ks_random_setup(e, norandomexits)
     end
 end
 
--- the Serpent should appear in exactly one hell end
--- XXX: are things like shafts going to break this?
-function hell_branches_remaining()
-   local hell_branches = { "Geh", "Coc", "Dis", "Tar" }
-   local ret = #hell_branches
-   for _, branch in ipairs(hell_branches) do
-      if travel.find_deepest_explored(branch) == 7 then
-         ret = ret - 1
-      end
-   end
-   return ret
+function soh_hangout()
+  if dgn.persist.soh_hangout == nil then
+    local hell_branches = { "Geh", "Coc", "Dis", "Tar" }
+    dgn.persist.soh_hangout = util.random_from(hell_branches)
+  end
+  return dgn.persist.soh_hangout
 end
 
+-- the Serpent should appear in exactly one hell end. Vaults that call this
+-- should guarantee that `D` is present.
 function serpent_of_hell_setup(e)
-   if not you.uniques("the Serpent of Hell") and
-      crawl.one_chance_in(hell_branches_remaining()) then
-      e.kmons('D = the Serpent of Hell')
-   end
+  local b = soh_hangout()
+  if not you.uniques("the Serpent of Hell")
+        and you.in_branch(b)
+        and you.depth() == dgn.br_depth(b) then
+    e.kmons('D = the Serpent of Hell')
+  end
 end
 
--- Guarantee two rare base types with a brand
+-- Guarantee two or three rare base types with a brand
 function hall_of_blades_weapon(e)
   local long_blade_type = crawl.one_chance_in(2) and "double sword"
                                                   or "triple sword"
   local types = {"quick blade", long_blade_type,
                  "executioner's axe", "eveningstar", "bardiche",
                  "lajatang"}
-  local egos = {"flaming", "freezing", "electrocution", "venom",
-              "holy_wrath", "pain", "vampirism", "draining",
-              "antimagic", "distortion"}
+  local egos = {"flaming", "freezing", "electrocution", "heavy",
+                "holy_wrath", "pain", "vampirism",
+                "antimagic", "distortion", "spectral"}
   local weapon1 = util.random_from(types)
   local weapon2 = weapon1
+  local weapon3 = weapon1
   while weapon2 == weapon1 do
     weapon2 = util.random_from(types)
   end
+  while weapon3 == weapon1 or weapon3 == weapon2 do
+    weapon3 = util.random_from(types)
+  end
   local ego1 = util.random_from(egos)
   local ego2 = ego1
+  local ego3 = ego1
   while ego2 == ego1 do
     ego2 = util.random_from(egos)
   end
-
+  while ego3 == ego1 or ego3 == ego2 do
+    ego3 = util.random_from(egos)
+  end
   e.mons("dancing weapon; good_item " .. weapon1 .. " ego:" .. ego1)
   e.mons("dancing weapon; good_item " .. weapon2 .. " ego:" .. ego2)
+  e.mons("dancing weapon; good_item " .. weapon3 .. " ego:" .. ego3 .. " / nothing")
 end
 
 -- Setup for door vaults to define a common loot set and create the door
@@ -147,7 +154,7 @@ function door_vault_setup(e)
   e.lua_marker('+',  props_marker { connected_exclude="true" })
   e.lua_marker('a',
                props_marker { stop_explore="strange structure made of doors" })
-  e.kfeat("a = runed_door")
+  e.kfeat("a = runed_clear_door")
 end
 
 --[[
@@ -159,14 +166,14 @@ resistance, so it has all of the elemental resistances.
 @string glyph The glyph on which to define the KMONS.
 ]]
 function master_elementalist_setup(e, glyph, ele_staff)
-    local equip_def = " ; elemental staff . robe good_item"
+    local equip_def = " ; elemental staff . robe ego:willpower good_item"
     -- Don't want to use the fallback here, so we can know to give resistance
     -- ego robe if the elemental staff isn't available.
     if you.unrands("elemental staff") then
         equip_def = " ; staff of air . robe ego:resistance good_item"
     end
 
-    e.kmons(glyph .. " = wizard hd:18 name:master_elementalist n_rpl" ..
+    e.kmons(glyph .. " = occultist hd:18 name:master_elementalist n_rpl" ..
         " n_des n_noc tile:mons_master_elementalist" ..
         " spells:lehudib's_crystal_spear.11.wizard;" ..
             "chain_lightning.11.wizard;" ..
